<?php

namespace Teknisa\CTB\Services;
use Teknisa\Model\Geral\Entities\Prodfili;
use Teknisa\CTB\Factories\Ger02900ParametrosContabilizacaoFactory;
use Teknisa\Libs\Util\Environment;

class Ger02900ParametrosContabilizacaoService {

    protected $entityManager;
    protected $environment;
    protected $nrorg;
    protected $parametrosFactory;

    public function __construct(\Doctrine\ORM\EntityManager $entityManager, Environment $environment, Ger02900ParametrosContabilizacaoFactory $parametrosFactory) {
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->nrorg = $environment->getNrorgTrab();
        $this->parametrosFactory = $parametrosFactory;
    }

    public function ctbprdtpoperIsNew($row) {
        $ctbprdtpoper = $this->parametrosFactory->getCtbprdtpoper(array(
            "IDENTRSAIDOP" => $row['IDENTRSAIDOP'],
            "CDTIPOOPER" => $row['CDTIPOOPER'],
            "CDPRODUTO" => $row['CDPRODUTO'],
            "NRORG" => $this->nrorg
        ));
        return $ctbprdtpoper === false;
    }

    public function paramimpostofilIsNew($row) {
        $paramimpostofil = $this->parametrosFactory->getParamimpostofil(array(
            "CDIMPOSTO" => $row['CDIMPOSTO'],
            "CDFILIAL" => $row['CDFILIAL'],
            "IDENTRSAIDOP" => $row['IDENTRSAIDOP'],
            "CDTIPOOPER" => $row['CDTIPOOPER'],
            "NRORG" => $this->nrorg
        ));
        return $paramimpostofil === false;
    }

    public function validateParamimpostofilFilter($filter) {
        if (empty($filter["CDFILIAL"])) {
            throw new \Exception("É necessário filtrar por unidade.");
        }
        if (empty($filter["IDENTRSAIDOP"])) {
            throw new \Exception("É necessário filtrar por tipo.");
        }
        if (empty($filter["CDTIPOOPER"])) {
            throw new \Exception("É necessário filtrar por tipo de operação.");
        }
    }

    public function ctbimptpoperIsNew($row) {
        $ctbimptpoper = $this->parametrosFactory->getCtbimptpoper(array(
            "IDENTRSAIDOP" => $row['IDENTRSAIDOP'],
            "CDTIPOOPER" => $row['CDTIPOOPER'],
            "CDIMPOSTO" => $row['CDIMPOSTO'],
            "NRORG" => $this->nrorg
        ));
        return $ctbimptpoper === false;
    }

    public function validateCtbimptpoperFilter($filter) {
        if (empty($filter["IDENTRSAIDOP"])) {
            throw new \Exception("É necessário filtrar por tipo.");
        }
        if (empty($filter["CDTIPOOPER"])) {
            throw new \Exception("É necessário filtrar por tipo de operação.");
        }
    }

    public function prodfiliIsNew($row) {
        $prodfili = $this->parametrosFactory->getProdfili(array(
            "CDFILIAL" => $row['CDFILIAL'],
            "CDPRODUTO" => $row['CDPRODUTO'],
            "NRORG" => $this->nrorg
        ));
        return $prodfili === false;
    }

    public function validateProdfiliFilter($filter) {
        if (empty($filter["CDFILIAL"])) {
            throw new \Exception("É necessário filtrar por unidade.");
        }
    }

    public function validarTipoctpagar($row) {
        if ($row["IDCTBLCTPAG"] == 'S' && $row["IDCTBTPCNTBPAG"] == 'S') {
            throw new \Exception("Não é possivel contabilizar no lançamento ao mesmo tempo que na baixa.");
        }
    }

    public function tipoctpagarfiliIsNew($row) {
        $tipoctpagarfili = $this->parametrosFactory->getTipoctpagarfili(array(
            "CDFILIAL" => $row['CDFILIAL'],
            "CDTPCTPAGAR" => $row['CDTPCTPAGAR'],
            "NRORG" => $this->nrorg
        ));
        return $tipoctpagarfili === false;
    }

    public function validarTipoctreceber($row) {
        if ($row["IDCTBLCTREC"] == 'S' && $row["IDCTBTPCNTBREC"] == 'S') {
            throw new \Exception("Não é possivel contabilizar no lançamento ao mesmo tempo que na baixa.");
        }
    }

    public function cadastramentoAutomaticoProdutos($row) {
        $prodfiliRepository = $this->entityManager->getRepository(Prodfili::class);
        if ($row["IDTIPOCAD"] == 'P') {
            if ($row['CDPRODINI'] > $row['CDPRODFIM']) {
                throw new \Exception("O produto final deve ser maior que o produto inicial.");
            }
            $produtos = $this->parametrosFactory->getProdutosFiltrados(array(
                "CDPRODINI" => $row['CDPRODINI'],
                "CDPRODFIM" => $row['CDPRODFIM'],
                "NRORG" => $this->nrorg
            ));
            foreach ($produtos as $produto) {
                $prodfili = $prodfiliRepository->findOneBy(array(
                    "cdproduto" => $produto["CDPRODUTO"],
                    "cdfilial" => $row["CDFILIALPROD"],
                    "nrorg" => $this->nrorg
                ));
                if ($prodfili !== null) {
                    $prodfili->setCdctblentrpf($row["CDCTBLENTRPF"]);
                    $prodfili->setCdctblsaidpf($row["CDCTBLSAIDPF"]);
                    $prodfili->setCdctblprodpf($row["CDCTBLPRODPF"]);
                    $prodfili->setCdctblcpropf($row["CDCTBLCPROPF"]);
                    $prodfili->setCdctblpracpf($row["CDCTBLPRACPF"]);
                    $prodfili->setCdctblajuspf($row["CDCTBLAJUSPF"]);
                    $prodfili->setCdctblimplpf($row["CDCTBLIMPLPF"]);
                    $this->entityManager->persist($prodfili);
                }
            }
            $this->entityManager->flush();
        }
        else {
            if ($row["T_CDFILIALDEST"] == 'T') {
                $filiais = array();
                $filiaisFetch = $this->parametrosFactory->getFiliaisProdfiliFiltrados(array(
                    "CDFILIAL" => $row['CDFILIALORIG'],
                    "NRORG" => $this->nrorg
                ));
                foreach ($filiaisFetch as $filial) {
                    array_push($filiais, $filial["CDFILIAL"]);
                }
            }
            else {
                $filiais = $row["CDFILIALDEST"];
            }

            $produtos = $prodfiliRepository->findOneBy(array(
                "cdfilial" => $row["CDFILIALORIG"],
                "nrorg" => $this->nrorg
            ));

            $currentTime = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
            $operador = $this->environment->getCdOperador();
            foreach ($filiais as $filial) {
                foreach ($produtos as $produto) {
                    $prodfili = $prodfiliRepository->findOneBy(array(
                        "cdproduto" => $produto->getCdproduto(),
                        "cdfilial" => $filial,
                        "nrorg" => $this->nrorg
                    ));
                    if ($prodfili !== null) {
                        $prodfili->setCdctblentrpf($row["IDCTBLENTRPF"] === 'R' ? $produto->getCdctblentrpf() : $row["CDCTBLENTRPF"]);
                        $prodfili->setCdctblsaidpf($row["IDCTBLSAIDPF"] === 'R' ? $produto->getCdctblsaidpf() : $row["CDCTBLSAIDPF"]);
                        $prodfili->setCdctblprodpf($row["IDCTBLPRODPF"] === 'R' ? $produto->getCdctblprodpf() : $row["CDCTBLPRODPF"]);
                        $prodfili->setCdctblcpropf($row["IDCTBLCPROPF"] === 'R' ? $produto->getCdctblcpropf() : $row["CDCTBLCPROPF"]);
                        $prodfili->setCdctblpracpf($row["IDCTBLPRACPF"] === 'R' ? $produto->getCdctblpracpf() : $row["CDCTBLPRACPF"]);
                        $prodfili->setCdctblajuspf($row["IDCTBLAJUSPF"] === 'R' ? $produto->getCdctblajuspf() : $row["CDCTBLAJUSPF"]);
                        $prodfili->setCdctblimplpf($row["IDCTBLIMPLPF"] === 'R' ? $produto->getCdctblimplpf() : $row["CDCTBLIMPLPF"]);
                        $prodfili->setNrorgultatu($this->nrorg);
                        $prodfili->setCdoperultatu($operador);
                        $prodfili->setDtultatu($currentTime);
                        $this->entityManager->persist($prodfili);
                    }
                }
            }
            $this->entityManager->flush();
        }
    }

    public function cadastramentoAutomaticoTitulos($row) {
        if ($row["T_CDFILIALDEST"] == 'T') {
            $filiais = array();
            $filiaisFetch = $this->parametrosFactory->getFiliaisTipoctpagarfiliFiltrados(array(
                "CDFILIAL" => $row['CDFILIALORIG'],
                "NRORG" => $this->nrorg
            ));
            foreach ($filiaisFetch as $filial) {
                array_push($filiais, $filial["CDFILIAL"]);
            }
        }
        else {
            $filiais = $row["CDFILIALDEST"];
        }

        $tiposTitulosPagar = $this->parametrosFactory->getTipoctpagarfiliFiltrados(array(
            "CDFILIAL" => $row['CDFILIALORIG'],
            "NRORG" => $this->nrorg
        ));

        $currentTime = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
        $operador = $this->environment->getCdOperador();
        foreach ($filiais as $filial) {
            foreach ($tiposTitulosPagar as $tipoTituloPagar) {
                $params = array("CDFILIAL" => $filial, "CDTPCTPAGAR" => $tipoTituloPagar["CDTPCTPAGAR"]);
                if ($this->tipoctpagarfiliIsNew($params)) {
                    $this->parametrosFactory->insertTipoctpagarfili(array(
                        "CDFILIAL" => $filial,
                        "CDTPCTPAGAR" => $tipoTituloPagar["CDTPCTPAGAR"],
                        "CDCONTCTBL" => $tipoTituloPagar["CDCONTCTBL"],
                        "NRORG" => $this->nrorg,
                        "DTINCLUSAO" => $currentTime,
                        "NRORGINCLUSAO" => $this->nrorg,
                        "CDOPERINCLUSAO" => $operador
                    ));
                }
                else {
                    $this->parametrosFactory->updateTipoctpagarfili(array(
                        "CDFILIAL" => $filial,
                        "CDTPCTPAGAR" => $tipoTituloPagar["CDTPCTPAGAR"],
                        "CDCONTCTBL" => $tipoTituloPagar["CDCONTCTBL"],
                        "NRORG" => $this->nrorg,
                        "DTULTATU" => $currentTime,
                        "NRORGULTATU" => $this->nrorg,
                        "CDOPERULTATU" => $operador
                    ));
                }
            }
        }
    }
}
